import java.util.Random;

public class StudentMazeGenerator implements MazeGenerator
{
    private Random random = new Random();
    protected Maze maze;
    protected MazeGeneratorListener listener;
            
	public Maze generateMaze(int width, int height, MazeGeneratorListener listener)
	{
        // Create the initial maze data structure.
        maze = MazeFactory.createMaze(width, height);
        // This is optional: for observing changes during maze generation.
        (this.listener = listener).startingMazeGeneration(maze);
        // Generate a full size maze (starting at position (0,0) with the given
        // width and height.
        // Splitting starts randomly (either verical or horizontal).
        generateMaze(0, 0, width, height, random.nextBoolean());
        return maze;
    }
    
    // Generate a maze of size width times height at position (offsetx, offsety)
    // in the maze data structure.
    // Do it by splitting the maze. vertical indicate whether the maze is split
    // verticall or horizontally.
	public void generateMaze(int offsetx, int offsety, int width, int height, boolean vertical)
	{
        // If the maze is of minimal size, there is nothing to do.
        if (width == 1 && height == 1) {
            return;
        }

        // If split should be done vertically (and it can be done vertically)
        if (vertical && width > 1) {
            // Compute the new widths for the two mazes (results of split)
            int width1 = nextPosNumber(width - 1) + 1;
            int width2 = width - width1;
            
            // Recursion: divide (generate left and right maze)
            generateMaze(offsetx, offsety, width1, height, 
                         // In order to get a good lucking maze,
                         // we always split the side which is longer
                         // (or horizontally if equal...)
                         width1 > height);
            // Adjust start position (x-offsets):
            // maze on right hand side start at the old position plus widht1...
            generateMaze(offsetx + width1, offsety, width2, height, width2 > height);
            // Now we have the right and the left maze attached to each other
            // but they are separated by a wall. So dig a whole in the wall (at a
            // random position) and we get a correct joint maze.
            maze.removeWallWestOf(offsetx + width1, offsety + nextPosNumber(height));        
        }
        else {
            // Same as vertical case, just roles of x and y exchanged.
            int height1 = nextPosNumber(height - 1) + 1;
            int height2 = height - height1;
            
            generateMaze(offsetx, offsety, width, height1, width > height1);
            generateMaze(offsetx, offsety + height1, width, height2, width > height2);
            
            maze.removeWallNorthOf(offsetx + nextPosNumber(width), offsety + height1);
        }
        // Tell the GUI that the maze has changed a little...
        listener.mazeModified(maze);
	}
    
    // Get a randomized positive number between 0 and range - 1
    private int nextPosNumber(int range) 
    {
        if (range == 0) return 0;
        
        int r = random.nextInt();
        
        return (r > 0 ? r : -r) % range;
    }
    

    /*
    private void printMaze(Maze maze, java.io.PrintStream output)
    {
        for (int x = 0; x < maze.getWidth(); x++) {
            output.print(" _");
        }
        output.println();
        
        for (int y = 0; y < maze.getHeight(); y++) {
            if (!maze.canMoveWestFrom(0, y)) {
                output.print('|');
            }
            for (int x = 0; x < maze.getWidth(); x++) {
                if (!maze.canMoveSouthFrom(x, y)) {
                    output.print('_');
                }
                else {
                    output.print(' ');
                }
                
                if (!maze.canMoveEastFrom(x, y)) {
                    output.print('|');
                }
                else {
                    output.print(' ');
                }
            }
            output.println();
        }
    }
    */
}
