import java.util.Vector;
import java.util.Set;
import java.util.HashSet;


// The StudentMazeSolver class is where you will implement your maze
// solving algorithm.  Presently, it returns a newly-created solution
// which contains only the starting position (and nothing else).

public class StudentMazeSolver implements MazeSolver
{
    private Maze maze;
    private MazeSolverListener listener;
    private NodeQueue nodeQueue;
    private Node exitNode;
    private Set visited;
    
    public MazeSolution solveMaze(Maze maze, MazeSolverListener listener)
    {
        
        this.maze = maze;
        this.listener = listener;
        nodeQueue = new NodeQueue();
        
        Node startNode = new Node(maze.getStartXPosition(), maze.getStartYPosition(), null);
        nodeQueue.enqueue(startNode);
        visited = new HashSet();
        exitNode = new Node(maze.getEndXPosition(), maze.getEndYPosition(), null);
        listener.startingMazeSolution(getCurrentSolution(null));
        
        return doBreadthSearch();
    }

    // Implement me!
    private MazeSolution doBreadthSearch() {
        return null;
    }

    
    /**
     * Transform a list of nodes into a MazeSolution object. The list of nodes should
     * represent a path from the entry of the maze to some other point in the maze. The list of
     * nodes must be backward chained meaning that the first node in list represents the
     * end of the path and the last node represents the actual entry position of the maze.
     * For a final maze solution the fist node in the list must correspond to the exit position
     * of the maze.
     * 
     * @param node The list of nodes (represented by the first node element). The nodes are chained
     * by the "previous" attribute in class Node. The first list element represents the end of
     * the solution path. If node is null, a default MazeSolution object is returned.
     * 
     * @return The MazeSolution object that corresponds to the node list.
     */
    private MazeSolution getCurrentSolution(Node node) {
        MazeSolution solution = MazeSolutionFactory.
            createMazeSolution(maze.getWidth(), maze.getHeight(),
                               maze.getStartXPosition(), maze.getStartYPosition(),
                               maze.getEndXPosition(), maze.getEndYPosition());
        
        Vector inv = new Vector();
        
        while (node != null) {
            inv.add(node);
            node = node.getPreviousNode();
        }
        
        for (int i = inv.size() - 2; i >= 0; i--) {
            node = (Node) inv.elementAt(i);
            Node previous = (Node) inv.elementAt(i + 1);
            
            int diffX = previous.getX() - node.getX();
            int diffY = previous.getY() - node.getY();
            
            if (diffX == 1) { solution.moveWest(); }
            else if (diffX == -1) { solution.moveEast(); }
            else if (diffY == 1) { solution.moveNorth(); }
            else if (diffY == -1) { solution.moveSouth(); }
        }
        
        return solution;
    }
}


class Node {
    private int x;
    private int y;
    
    private Node previousNode;
    
    public Node(int x, int y, Node previousNode) {
        this.x = x;
        this.y = y;
        this.previousNode = previousNode;
    }
    
    public int getX() { return x; }
    
    public int getY() { return y; }
    
    public Node getPreviousNode() { return previousNode; }
    
    // Nodes are considered to be equal on the mere basis of their coordinates
    // in the maze. The predecessor (attribute previousNode)
    // is not relevant in this respect.
    //
    // Overriding equals() and hashCode() allows for storing "visited" nodes
    // in a HashSet while using the "Set.contains()" method for checking if
    // a node has already been visited or not.
    //
    public boolean equals(Object o) {
        if (o == null || !(o instanceof Node)) return false;
        
        Node node = (Node) o;
        
        return node.getX() == getX() && node.getY() == getY();
    }
    
    
    public int hashCode() {
        return x + 100 * y;
    }
    
    
    public String toString() {
        return "Node: (" + x + ", " + y + ")";
    }
}


class NodeQueue {
    private Vector vector;
    
    public NodeQueue() {
        vector = new Vector();
    }
    
    public void enqueue(Node o) {
        vector.addElement(o);
    }
    
    public boolean isEmpty() {
        return vector.size() == 0;
    }
    
    public Node front() {
        return (Node) vector.elementAt(0);
    }
    
    
    public void dequeue() {
        vector.remove(front());
    }
}